package org.jbpm.examples.java;

import java.util.ArrayList;
import java.util.List;

import org.jbpm.api.Configuration;
import org.jbpm.api.ExecutionService;
import org.jbpm.api.HistoryService;
import org.jbpm.api.IdentityService;
import org.jbpm.api.ManagementService;
import org.jbpm.api.ProcessEngine;
import org.jbpm.api.RepositoryService;
import org.jbpm.api.TaskService;
import org.jbpm.test.Db;

public class JBPMRepository {

	private ProcessEngine processEngine = null;
	private RepositoryService repositoryService;
	private ExecutionService executionService;
	private ManagementService managementService;
	private TaskService taskService;
	private HistoryService historyService;
	private IdentityService identityService;

	/**
	 * registered deployments. registered deployments will be deleted
	 * automatically in the tearDown. This is a convenience function as each
	 * test is expected to clean up the DB.
	 */
	private List<String> registeredDeployments = new ArrayList<String>();

	public synchronized void createProcessEngine() {
		if (processEngine == null) {
			processEngine = Configuration.getProcessEngine();
			repositoryService = processEngine.get(RepositoryService.class);
			executionService = processEngine.getExecutionService();
			historyService = processEngine.getHistoryService();
			managementService = processEngine.getManagementService();
			taskService = processEngine.getTaskService();
			identityService = processEngine.getIdentityService();
		}
	}
	
	public void deleteRegisteredDeployments() {
		for (String deploymentId : registeredDeployments) {
			repositoryService.deleteDeploymentCascade(deploymentId);
		}
	}

	/**
	 * deploys the process, keeps a reference to the deployment and
	 * automatically deletes the deployment in the tearDown
	 */
	public String deployJpdlXmlString(String jpdlXmlString) {
		String deploymentDbid = repositoryService.createDeployment()
				.addResourceFromString("xmlstring.jpdl.xml", jpdlXmlString)
				.deploy();
		registerDeployment(deploymentDbid);
		return deploymentDbid;
	}

	/** registered deployments will be deleted in the tearDown */
	public void registerDeployment(String deploymentId) {
		registeredDeployments.add(deploymentId);
	}
	
	public String verifyDbClean() {
		String errorMsg = null;
		String recordsLeftMsg = Db.verifyClean(processEngine);
		if ((recordsLeftMsg != null) && (recordsLeftMsg.length() > 0)) {
			errorMsg = "database was not clean after test: " + recordsLeftMsg;
		}
		return errorMsg;
	}

	public ProcessEngine getProcessEngine() {
		return processEngine;
	}

	public void setProcessEngine(ProcessEngine processEngine) {
		this.processEngine = processEngine;
	}

	public RepositoryService getRepositoryService() {
		return repositoryService;
	}

	public void setRepositoryService(RepositoryService repositoryService) {
		this.repositoryService = repositoryService;
	}

	public ExecutionService getExecutionService() {
		return executionService;
	}

	public void setExecutionService(ExecutionService executionService) {
		this.executionService = executionService;
	}

	public ManagementService getManagementService() {
		return managementService;
	}

	public void setManagementService(ManagementService managementService) {
		this.managementService = managementService;
	}

	public TaskService getTaskService() {
		return taskService;
	}

	public void setTaskService(TaskService taskService) {
		this.taskService = taskService;
	}

	public HistoryService getHistoryService() {
		return historyService;
	}

	public void setHistoryService(HistoryService historyService) {
		this.historyService = historyService;
	}

	public IdentityService getIdentityService() {
		return identityService;
	}

	public void setIdentityService(IdentityService identityService) {
		this.identityService = identityService;
	}
}
